// Banco de pruebas de la calculadora

`timescale 1ns / 1ps

`define TP 20   // periodo de reloj

module test ();

    reg clk;
    reg reset;
    reg xs;
    reg i1, i0;
    wire finish;

    // Circuito bajo test
    calculator #(.W(8)) uut (
        .clk(clk), .reset(reset), .xs(xs), .i1(i1), .i0(i0), .finish(finish));

    // generador de relog
    always
        #(`TP/2) clk = ~clk;

    // prodedimiento de test principal
    initial begin
        $dumpfile("calculadora_tb.vcd");
        $dumpvars(0, test);

        // valores iniciales
        clk = 1'b0;
        reset = 1'b0;
        xs = 1'b0;
        i1 = 0; i0 = 0;

        // contenido inicial de A y B
        uut.data_path.a = 8'd5;
        uut.data_path.b = 8'd7;

        // reset inicial
        @(posedge clk)
        #1 reset = 1'b1;
        @(posedge clk)
        #1 reset = 1'b0;

        // A <- A + B (A = 5 + 7)

        // activamos la calculadora y seleccionamos la operación
        @(posedge clk)
        #1 xs = 1'b1;
        i1 = 1'b0; i0 = 1'b0;
        @(posedge clk)
        #1 xs = 1'b0;

        // esperamos 5 ciclos a que calcule el resultado
        repeat(5)
            @(posedge clk);

        // A debe ser igual a 12
        if (uut.data_path.a != 12)
            $display("ERROR: A = %d, esperaba 12", uut.data_path.a);

        // B <- A - B (B = 12 - 7)

        // activamos la calculadora y seleccionamos la operación
        @(posedge clk)
        #1 xs = 1'b1;
        i1 = 1'b1; i0 = 1'b1;
        @(posedge clk)
        #1 xs = 1'b0;

        // esperamos 5 ciclos a que calcule el resultado
        repeat(5)
            @(posedge clk);

        // B debe ser igual a 5
        if (uut.data_path.b != 5)
            $display("ERROR: B = %d, esperaba 5", uut.data_path.b);

        // finalizamos la simulación
        $finish;
    end
endmodule

/*
   EJERCICIOS

   1. Compila y simula el diseño con:

      $ iverilog calculadora.v calculadora_tb.v
      $ vvp a.out

      Resuelve cualquier posible error de sintaxis o de simulación que ocurra.

   2. Visualiza los resultados con:

      $ gtkwave calculadora_tb.vcd &

      Observa las señales clk, reset, xs, i1, i0 y finish. Comprueba que se
      activan de acuerdo a lo esperado.

      Añade a la visualización el contenido de los registros a y b, las
      señales de control de los registros a y b, el estado de la unidad de
      control y el contenido del bus de la unidad de datos. Comprueba las
      transiciones de cada una de estas señales hasta que comprendas por qué se
      produce cada una de ellas.

   3. Amplía el banco de pruebas para simular las operaciones A<-A-B y B<-A+B.
      Comprueba que la operación es correcta.

   4. Usando este diseño como referencia, realice un diseño Verilog y banco de
      pruebas de la calculadora de tres buses del tema. Compruebe su correcto
      funcionamiento.
 */
